<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Smtp_Mailer' ) ) {
	/**
	 * Send Mail using SMTP
	 *
	 * Class ES_Smtp_Mailer
	 *
	 * @since 4.2.1
	 */
	class ES_Smtp_Mailer extends ES_Base_Mailer {

		/**
		 * Mailer name
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $name = 'SMTP';

		/**
		 * Mailer Slug
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $slug = 'smtp';

		/**
		 * ES_Smtp_Mailer constructor.
		 *
		 * @since 4.2.1
		 */
		public function __construct() {
			parent::__construct();
		}

		/**
		 * Send Mail using SMTP
		 *
		 * @param ES_Message $message
		 *
		 * @return bool|WP_Error
		 *
		 * @since 4.2.1
		 * @since 4.3.2 Modified arguments.
		 */
		public function send( ES_Message $message ) {

			ES()->logger->info( 'Start Sending Email Using SMTP', $this->logger_context );

			$phpmailer = ES()->mailer->get_phpmailer();
			
			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );

			if ( ES()->is_const_defined( 'smtp', 'host' ) ) {
				$smtp_host = ES()->get_const_value( 'smtp', 'host' );
			} else {
				$smtp_host = ! empty( $ig_es_mailer_settings['smtp']['smtp_host'] ) ? $ig_es_mailer_settings['smtp']['smtp_host'] : '';
			}

			if ( empty( $smtp_host ) ) {
				return $this->do_response( 'error', __( 'SMTP Host is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'smtp', 'encryption' ) ) {
				$smtp_encryption = ES()->get_const_value( 'smtp', 'encryption' );
			} else {
				$smtp_encryption = ! empty( $ig_es_mailer_settings['smtp']['smtp_encryption'] ) ? $ig_es_mailer_settings['smtp']['smtp_encryption'] : '';
			}

			if ( empty( $smtp_encryption ) ) {
				return $this->do_response( 'error', __( 'SMTP Encryption is not set.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'smtp', 'port' ) ) {
				$smtp_port = ES()->get_const_value( 'smtp', 'port' );
			} else {
				$smtp_port = ! empty( $ig_es_mailer_settings['smtp']['smtp_port'] ) ? $ig_es_mailer_settings['smtp']['smtp_port'] : '';
			}

			if ( empty( $smtp_port ) ) {
				return $this->do_response( 'error', __( 'SMTP Port is empty.', 'email-subscribers' ) );
			}
			
			if ( ES()->is_const_defined( 'smtp', 'authentication' ) ) {
				$smtp_authentication = ES()->get_const_value( 'smtp', 'authentication' );
			} else {
				$smtp_authentication = ! empty( $ig_es_mailer_settings['smtp']['smtp_authentication'] ) ? $ig_es_mailer_settings['smtp']['smtp_authentication'] : '';
			}

			if ( empty( $smtp_authentication ) ) {
				return $this->do_response( 'error', __( 'SMTP Authentication is empty.', 'email-subscribers' ) );
			}
			
			if ( ES()->is_const_defined( 'smtp', 'username' ) ) {
				$smtp_username = ES()->get_const_value( 'smtp', 'username' );
			} else {
				$smtp_username = ! empty( $ig_es_mailer_settings['smtp']['smtp_username'] ) ? $ig_es_mailer_settings['smtp']['smtp_username'] : '';
			}

			if ( 'yes' == $smtp_authentication && empty( $smtp_username ) ) {
				return $this->do_response( 'error', __( 'SMTP Username is empty.', 'email-subscribers' ) );
			}
			
			if ( ES()->is_const_defined( 'smtp', 'password' ) ) {
				$smtp_password = ES()->get_const_value( 'smtp', 'password' );
			} else {
				$smtp_password = ! empty( $ig_es_mailer_settings['smtp']['smtp_password'] ) ? $ig_es_mailer_settings['smtp']['smtp_password'] : '';
			}

			if ( 'yes' == $smtp_authentication &&  empty( $smtp_password ) ) {
				return $this->do_response( 'error', __( 'SMTP Password is empty.', 'email-subscribers' ) );
			}

			$smtp_authentication = ( 'yes' === $smtp_authentication ) ? true : false;

			$phpmailer->isSMTP();
			$phpmailer->SMTPKeepAlive = true;
			$phpmailer->Host          = $smtp_host;
			$phpmailer->SMTPAuth      = $smtp_authentication;
			$phpmailer->Username      = $smtp_username;
			$phpmailer->Password      = $smtp_password;
			$phpmailer->SMTPSecure    = $smtp_encryption;
			$phpmailer->Port          = $smtp_port;
			$phpmailer->From          = $message->from;
			$phpmailer->FromName      = $message->from_name;

			$phpmailer->ClearAllRecipients();
			$phpmailer->clearAttachments();
			$phpmailer->clearCustomHeaders();
			$phpmailer->clearReplyTos();

			$phpmailer->addAddress( $message->to );
			$phpmailer->addReplyTo( $message->reply_to_email );

			$phpmailer->WordWrap = 50;
			$phpmailer->isHTML( true );

			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $message->to );
			if ( ! empty( $list_unsubscribe_header ) ) {
				$phpmailer->addCustomHeader( 'List-Unsubscribe', $list_unsubscribe_header );
				$phpmailer->addCustomHeader( 'List-Unsubscribe-Post', 'List-Unsubscribe=One-Click' );
			}

			apply_filters( 'ig_es_smtp_mailer_email_headers', $phpmailer );

			$phpmailer->Subject = $message->subject;
			$phpmailer->Body    = $message->body;
			$phpmailer->AltBody = $message->body_text; //Text Email Body for non html email client

			$attachments = $message->attachments;
			if ( ! empty( $attachments ) ) {
				foreach ( $attachments as $attachment ) {
					try {
						$phpmailer->addAttachment( $attachment );
					} catch ( phpmailerException $e ) {
						continue;
					}
				}
			}
			
			try {
				if ( ! $phpmailer->send() ) {
					ES()->logger->error( '[Error in Email Sending] : ' . $message->to . ' Error: ' . $phpmailer->ErrorInfo, $this->logger_context );

					return $this->do_response( 'error', $phpmailer->ErrorInfo );
				}
			} catch ( Exception $e ) {
				ES()->logger->error( '[Error in Email Sending] : ' . $message->to . ' Error: ' . $e->getMessage(), $this->logger_context );

				return $this->do_response( 'error', $e->getMessage() );
			}

			ES()->logger->info( 'Email sent successfully using SMTP', $this->logger_context );

			return $this->do_response( 'success' );
		}
	}
}
