<?php

defined( 'ABSPATH' ) || exit;

/***
 * ES_Trigger_Ninja_Forms_Submitted class.
 *
 * @since 4.4.1
 */
class ES_Trigger_Ninja_Forms_Submitted extends ES_Workflow_Trigger {

	/**
	 * Declares data items available in trigger.
	 *
	 * @var array
	 */
	public $supplied_data_items = array( 'ninja_forms_data' );

	/**
	 * Load trigger admin props.
	 */
	public function load_admin_details() {
		$this->title       = __( 'Ninja Form Submitted', 'email-subscribers' );
		$this->description = __( 'Fires whenever someone fill up Ninja Forms.', 'email-subscribers' );
		$this->group       = __( 'Form', 'email-subscribers' );
	}

	/**
	 * Load fields.
	 */
	public function load_fields() {

		// targeting database class provided in api to get form data.
		if ( class_exists( 'NF_Database_FormsController' ) && method_exists( NF_Database_FormsController::class, 'getFormsData' ) ) {

			$forms = array(
				0 => __( 'Any Forms', 'email-subscribers' ),
			);

			$ninja_forms_instance = new NF_Database_FormsController();
			$ninja_forms          = $ninja_forms_instance->getFormsData();

			if ( ! empty( $ninja_forms ) && is_array( $ninja_forms ) ) {

				foreach ( $ninja_forms as $form ) {
					$form_title = $form->title;
					$form_id    = $form->id;

					if ( ! empty( $form_id ) && ! empty( $form_title ) ) {

						if ( mb_strlen( $form_title ) > 25 ) {
							$form_title = mb_substr( $form_title, 0, 25 ) . '...';
						}

						/* translators: %s: Ninja form form ID. */
						$form_title = $form_title . ' ' . sprintf( esc_html__( '(ID: %s)', 'email-subscribers' ), $form_id );

						$forms[ $form_id ] = $form_title;
					}
				}

				$form_list_field = new ES_Select();
				$form_list_field->set_name( 'ig-es-form-id' );
				$form_list_field->set_title( __( 'Select Forms ', 'email-subscribers' ) );
				$form_list_field->set_options( $forms );
				$form_list_field->set_required();
				$this->add_field( $form_list_field );
			}
		}

	}

	/**
	 * Register trigger hooks.
	 */
	public function register_hooks() {
		add_action( 'ninja_forms_after_submission', array( $this, 'handle_ninja_form_submission' ), 20, 1 );
	}

	/**
	 * Catch Ninja Forms submission hook.
	 *
	 * @param $data
	 */
	public function handle_ninja_form_submission( $data ) {

		if ( ! empty( $data ) ) {

			$fields = (array) $data['fields'];

			if ( count( $fields ) > 0 ) {

				$name    = '';
				$email   = '';
				$form_id = '';

				if ( ! empty( $data['form_id'] ) ) {
					$form_id = $data['form_id'];
				}

				foreach ( $fields as $field ) {
					if ( ! empty( $field['type'] ) && 'email' === $field['type'] ) {
						$email = ! empty( $field['value'] ) ? sanitize_email( $field['value'] ) : '';
					}

					// By default, we are checking this field.
					if ( ! empty( $field['type'] ) && 'textbox' === $field['type'] && 'Name' === $field['label'] ) {
						$name = ! empty( $field['value'] ) ? sanitize_text_field( $field['value'] ) : '';
					} elseif ( ! empty( $field['type'] ) && 'firstname' === $field['type'] && 'First Name' === $field['label'] ) {
						$name = ! empty( $field['value'] ) ? sanitize_text_field( $field['value'] ) : '';
					}
				}

				if ( ! empty( $email ) && ! empty( $form_id ) ) {

					$data = array(
						'ninja_forms_data' => array(
							'form_id' => $form_id,
							'name'    => $name,
							'email'   => $email,
						),
					);

					$this->maybe_run( $data );
				}
			}
		}
	}


	/**
	 * Validate a workflow.
	 *
	 * @param ES_Workflow $workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {

		$ninja_forms_data = $workflow->data_layer()->get_item( 'ninja_forms_data' );

		if ( ! is_array( $ninja_forms_data ) || empty( $ninja_forms_data['email'] ) ) {
			return false;
		}

		$selected_form_id = $workflow->get_trigger_option( 'ig-es-form-id' );

		if ( ! empty( $selected_form_id ) && $selected_form_id !== $ninja_forms_data['form_id'] ) {
			return false;
		}

		return true;
	}

}
